#include <math.h>    
#include "trigfun.h"  
/* 
 *  
 *            Syntax  [sys, x0] = Network11bus_ActualModel(t,x,u,flag,X0) 
*/ 
/* 
* The following #define is used to specify the name of this S-Function. 
*/ 
#define S_FUNCTION_NAME Network11bus_ActualModel 
/* 
* need to include simstruc.h for the definition of the SimStruct and 
* its associated macro definitions. 
*/ 
#include "simstruc.h" 
  
/* Defines for work arrays */ 
#define Br(j) RWork[j] 
#define Ar(i,j) RWork[22+ i+j*22] 
#define Cr(i,j) RWork[506+ i+j*22] 
/* 
 * mdlInitializeSizes - initialize the sizes array 
 * 
 * The sizes array is used by SIMULINK to determine the S-function block's 
 * characteristics (number of inputs, outputs, states, etc.). 
 */ 
static void mdlInitializeSizes(S) 
    SimStruct *S; 
{ 
    ssSetNumContStates(    S, 0);      /* number of continuous states */ 
    ssSetNumDiscStates(    S, 22);      /* number of discrete states */ 
    ssSetNumInputs(        S, 22);      /* number of inputs */ 
    ssSetNumOutputs(       S, 22);      /* number of outputs */ 
    ssSetDirectFeedThrough(S, 0);      /* direct feedthrough flag */ 
    ssSetNumSampleTimes(   S, 1);      /* number of sample times */ 
    ssSetNumInputArgs(     S, 1);      /* number of input arguments */ 
    ssSetNumRWork(         S, 990);      /* number of real work vector elements */ 
    ssSetNumIWork(         S, 0);      /* number of integer work vector elements */ 
    ssSetNumPWork(         S, 0);      /* number of pointer work vector elements */ 
     /* 
     * if there aren't the correct number of parameters, just return, simulink.c will 
     * flag the error 
     */ 
    if (ssGetNumArgs(S) != 1) 
       return; 
    if (mxGetM(ssGetArg(S,0)) != 22  || mxGetN(ssGetArg(S,0)) !=1)  { 
        ssSetErrorStatus( S, "X0 must have dimensions: 22 by 1"); 
        return; 
    } 
} 
/* 
 * mdlInitializeSampleTimes - initialize the sample times array 
 * 
 * This function is used to specify the sample time(s) for your S-function. 
 * If your S-function is continuous, you must specify a sample time of 0.0. 
 * Sample times must be registered in ascending order. 
 */ 
static void mdlInitializeSampleTimes(S) 
    SimStruct *S; 
{ 
    ssSetSampleTimeEvent(S, 0, -1); 
    ssSetOffsetTimeEvent(S, 0, 0.0); 
} 
/* 
 * mdlInitializeConditions - initialize the states 
 * 
 * In this function, you should initialize the continuous and discrete 
 * states for your S-function block.  The initial states are placed 
 * in the x0 variable.  You can also perform any other initialization 
 * activities that your S-function may require. 
 */ 
static void mdlInitializeConditions(x0, S) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double *x0; 
    SimStruct *S; 
{ 
    int i,imax; 
    double *X0pr; 
/* Set initial value for state vector */ 
    X0pr = mxGetPr(ssGetArg(S,0));  
    imax = mxGetM(ssGetArg(S,0)); 
    for(i=0;i<imax;i++){ 
        x0[i] = X0pr[i]; 
    } 
} 
#else    /* compiled for dSpace board? */ 
    real_T *x0; 
    SimStruct *S; 
{ 
    int_T i,imax; 
    real_T *X0pr; 
/* Set initial value for state vector */ 
    X0pr = mxGetPr(ssGetArg(S,0));  
    imax = mxGetM(ssGetArg(S,0)); 
    for(i=0;i<imax;i++){ 
        x0[i] = X0pr[i]; 
    } 
} 
#endif 
/* 
 * mdlUpdate - perform action at major integration time step 
 * 
 * This function is called once for every major integration time step. 
 * Discrete states are typically updated here, but this function is useful 
 * for performing any tasks that should only take place once per integration 
 * step. 
 */ 
static void mdlUpdate(x, u, S, tid) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double  *x, *u; 
    SimStruct *S; 
    int tid; 
{  
    int n; 
    double s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12, s13, s14, s15, s16, s17, s18; 
    double t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14, t15, t16, t17, t18, t19, t20, t21, t22, t23, t24, t25, t26, t27, t28, t29, t30, t31, t32, t33, t34, t35; 
    double tempx0, tempx1, tempx2, tempx3, tempx4, tempx5, tempx6, tempx7, tempx8, tempx9, tempx10, tempx11, tempx12, tempx13, tempx14, tempx15, tempx16, tempx17, tempx18, tempx19, tempx20, tempx21; 
    double *RWork = ssGetRWork(S); 
    int i,j; 
    char s='L'; 
    int nrhs, LDA, info; 
#else    /* compiled for dSpace board? */ 
    real_T *x; 
    const real_T *u; 
    SimStruct *S; 
    int_T tid; 
{  
    int_T n; 
    real_T s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12, s13, s14, s15, s16, s17, s18; 
    real_T t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14, t15, t16, t17, t18, t19, t20, t21, t22, t23, t24, t25, t26, t27, t28, t29, t30, t31, t32, t33, t34, t35; 
    real_T tempx0, tempx1, tempx2, tempx3, tempx4, tempx5, tempx6, tempx7, tempx8, tempx9, tempx10, tempx11, tempx12, tempx13, tempx14, tempx15, tempx16, tempx17, tempx18, tempx19, tempx20, tempx21; 
    real_T *RWork = ssGetRWork(S); 
    int_T i,j; 
    char s='L'; 
    int_T nrhs, LDA, info; 
#endif 
    n = 22; 
  for(i = 1;i <= 3;i++) 
  { 
    t1=pow(x[19],2.); 
    t2=pow(x[18],2.); 
    t3=pow(x[17],2.); 
    t4=pow(x[16],2.); 
    t5=pow(x[15],2.); 
    t6=pow(x[14],2.); 
    t7=pow(x[13],2.); 
    t8=pow(u[3],2.); 
    t9=pow(x[21],2.); 
    t10=pow(x[20],2.); 
    t11=pow(u[2],2.); 
    t12=cos(x[8]-x[10]); 
    t13=cos(x[5]-x[10]); 
    t14=cos(x[10]-x[12]); 
    t15=cos(x[7]-x[9]); 
    t16=cos(x[5]-x[9]); 
    t17=cos(x[9]-x[11]); 
    t18=cos(x[4]-x[8]); 
    t19=cos(x[4]-x[7]); 
    t20=cos(x[6]-x[12]); 
    t21=cos(x[6]-x[11]); 
    t22=cos(u[1]-x[5]); 
    t23=cos(u[0]-x[4]); 
    t24=sin(x[8]-x[10]); 
    t25=sin(x[5]-x[10]); 
    t26=-sin(x[10]-x[12]); 
    t27=sin(x[7]-x[9]); 
    t28=sin(x[5]-x[9]); 
    t29=-sin(x[9]-x[11]); 
    t30=sin(x[4]-x[8]); 
    t31=sin(x[4]-x[7]); 
    t32=-sin(x[6]-x[12]); 
    t33=-sin(x[6]-x[11]); 
    t34=sin(u[1]-x[5]); 
    t35=sin(u[0]-x[4]); 
    Ar(0,0)=1.; 
    Ar(0,1)=0; 
    Ar(0,2)=0; 
    Ar(0,3)=0; 
    Ar(0,4)=3.636*t23*u[2]*x[13]; 
    Ar(0,5)=0; 
    Ar(0,6)=0; 
    Ar(0,7)=0; 
    Ar(0,8)=0; 
    Ar(0,9)=0; 
    Ar(0,10)=0; 
    Ar(0,11)=0; 
    Ar(0,12)=0; 
    Ar(0,13)=-3.636*t35*u[2]; 
    Ar(0,14)=0; 
    Ar(0,15)=0; 
    Ar(0,16)=0; 
    Ar(0,17)=0; 
    Ar(0,18)=0; 
    Ar(0,19)=0; 
    Ar(0,20)=0; 
    Ar(0,21)=0; 
    Ar(1,0)=0; 
    Ar(1,1)=1.; 
    Ar(1,2)=0; 
    Ar(1,3)=0; 
    Ar(1,4)=0; 
    Ar(1,5)=3.636*t22*u[3]*x[14]; 
    Ar(1,6)=0; 
    Ar(1,7)=0; 
    Ar(1,8)=0; 
    Ar(1,9)=0; 
    Ar(1,10)=0; 
    Ar(1,11)=0; 
    Ar(1,12)=0; 
    Ar(1,13)=0; 
    Ar(1,14)=-3.636*t34*u[3]; 
    Ar(1,15)=0; 
    Ar(1,16)=0; 
    Ar(1,17)=0; 
    Ar(1,18)=0; 
    Ar(1,19)=0; 
    Ar(1,20)=0; 
    Ar(1,21)=0; 
    Ar(2,0)=0; 
    Ar(2,1)=0; 
    Ar(2,2)=0; 
    Ar(2,3)=0; 
    Ar(2,4)=x[13]*(-3.636*t23*u[2]+(-40.672*t19+0.40672*t31)*x[16]+(-40.672*t18+0.40672*t30)*x[17]); 
    Ar(2,5)=0; 
    Ar(2,6)=0; 
    Ar(2,7)=(40.672*t19-0.40672*t31)*x[13]*x[16]; 
    Ar(2,8)=(40.672*t18-0.40672*t30)*x[13]*x[17]; 
    Ar(2,9)=0; 
    Ar(2,10)=0; 
    Ar(2,11)=0; 
    Ar(2,12)=0; 
    Ar(2,13)=3.636*t35*u[2]+1.62688*x[13]+(-0.40672*t19-40.672*t31)*x[16]+(-0.40672*t18-40.672*t30)*x[17]; 
    Ar(2,14)=0; 
    Ar(2,15)=0; 
    Ar(2,16)=(-0.40672*t19-40.672*t31)*x[13]; 
    Ar(2,17)=(-0.40672*t18-40.672*t30)*x[13]; 
    Ar(2,18)=0; 
    Ar(2,19)=0; 
    Ar(2,20)=0; 
    Ar(2,21)=0; 
    Ar(3,0)=0; 
    Ar(3,1)=0; 
    Ar(3,2)=0; 
    Ar(3,3)=0; 
    Ar(3,4)=0; 
    Ar(3,5)=x[14]*(-3.636*t22*u[3]+(-40.672*t16+0.40672*t28)*x[18]+(-40.672*t13+0.40672*t25)*x[19]); 
    Ar(3,6)=0; 
    Ar(3,7)=0; 
    Ar(3,8)=0; 
    Ar(3,9)=(40.672*t16-0.40672*t28)*x[14]*x[18]; 
    Ar(3,10)=(40.672*t13-0.40672*t25)*x[14]*x[19]; 
    Ar(3,11)=0; 
    Ar(3,12)=0; 
    Ar(3,13)=0; 
    Ar(3,14)=3.636*t34*u[3]+1.62688*x[14]+(-0.40672*t16-40.672*t28)*x[18]+(-0.40672*t13-40.672*t25)*x[19]; 
    Ar(3,15)=0; 
    Ar(3,16)=0; 
    Ar(3,17)=0; 
    Ar(3,18)=(-0.40672*t16-40.672*t28)*x[14]; 
    Ar(3,19)=(-0.40672*t13-40.672*t25)*x[14]; 
    Ar(3,20)=0; 
    Ar(3,21)=0; 
    Ar(4,0)=0; 
    Ar(4,1)=0; 
    Ar(4,2)=0; 
    Ar(4,3)=0; 
    Ar(4,4)=0; 
    Ar(4,5)=0; 
    Ar(4,6)=x[15]*((-40.672*t21-0.40672*t33)*x[20]+(-40.672*t20-0.40672*t32)*x[21]); 
    Ar(4,7)=0; 
    Ar(4,8)=0; 
    Ar(4,9)=0; 
    Ar(4,10)=0; 
    Ar(4,11)=(40.672*t21+0.40672*t33)*x[15]*x[20]; 
    Ar(4,12)=(40.672*t20+0.40672*t32)*x[15]*x[21]; 
    Ar(4,13)=0; 
    Ar(4,14)=0; 
    Ar(4,15)=1.62688*x[15]+(-0.40672*t21+40.672*t33)*x[20]+(-0.40672*t20+40.672*t32)*x[21]; 
    Ar(4,16)=0; 
    Ar(4,17)=0; 
    Ar(4,18)=0; 
    Ar(4,19)=0; 
    Ar(4,20)=(-0.40672*t21+40.672*t33)*x[15]; 
    Ar(4,21)=(-0.40672*t20+40.672*t32)*x[15]; 
    Ar(5,0)=0; 
    Ar(5,1)=0; 
    Ar(5,2)=0; 
    Ar(5,3)=0; 
    Ar(5,4)=(40.672*t19+0.40672*t31)*x[13]*x[16]; 
    Ar(5,5)=0; 
    Ar(5,6)=0; 
    Ar(5,7)=x[16]*((-40.672*t19-0.40672*t31)*x[13]+(-20.32*t15+0.2032*t27)*x[18]); 
    Ar(5,8)=0; 
    Ar(5,9)=(20.32*t15-0.2032*t27)*x[16]*x[18]; 
    Ar(5,10)=0; 
    Ar(5,11)=0; 
    Ar(5,12)=0; 
    Ar(5,13)=(-0.40672*t19+40.672*t31)*x[16]; 
    Ar(5,14)=0; 
    Ar(5,15)=0; 
    Ar(5,16)=(-0.40672*t19+40.672*t31)*x[13]+1.21984*x[16]+(-0.2032*t15-20.32*t27)*x[18]; 
    Ar(5,17)=0; 
    Ar(5,18)=(-0.2032*t15-20.32*t27)*x[16]; 
    Ar(5,19)=0; 
    Ar(5,20)=0; 
    Ar(5,21)=0; 
    Ar(6,0)=0; 
    Ar(6,1)=0; 
    Ar(6,2)=0; 
    Ar(6,3)=0; 
    Ar(6,4)=(40.672*t18+0.40672*t30)*x[13]*x[17]; 
    Ar(6,5)=0; 
    Ar(6,6)=0; 
    Ar(6,7)=0; 
    Ar(6,8)=x[17]*((-40.672*t18-0.40672*t30)*x[13]+(-20.32*t12+0.2032*t24)*x[19]); 
    Ar(6,9)=0; 
    Ar(6,10)=(20.32*t12-0.2032*t24)*x[17]*x[19]; 
    Ar(6,11)=0; 
    Ar(6,12)=0; 
    Ar(6,13)=(-0.40672*t18+40.672*t30)*x[17]; 
    Ar(6,14)=0; 
    Ar(6,15)=0; 
    Ar(6,16)=0; 
    Ar(6,17)=(-0.40672*t18+40.672*t30)*x[13]+1.21984*x[17]+(-0.2032*t12-20.32*t24)*x[19]; 
    Ar(6,18)=0; 
    Ar(6,19)=(-0.2032*t12-20.32*t24)*x[17]; 
    Ar(6,20)=0; 
    Ar(6,21)=0; 
    Ar(7,0)=0; 
    Ar(7,1)=0; 
    Ar(7,2)=0; 
    Ar(7,3)=0; 
    Ar(7,4)=0; 
    Ar(7,5)=(40.672*t16+0.40672*t28)*x[14]*x[18]; 
    Ar(7,6)=0; 
    Ar(7,7)=(20.32*t15+0.2032*t27)*x[16]*x[18]; 
    Ar(7,8)=0; 
    Ar(7,9)=x[18]*((-40.672*t16-0.40672*t28)*x[14]+(-20.32*t15-0.2032*t27)*x[16]+(-20.32*t17-0.2032*t29)*x[20]); 
    Ar(7,10)=0; 
    Ar(7,11)=(20.32*t17+0.2032*t29)*x[18]*x[20]; 
    Ar(7,12)=0; 
    Ar(7,13)=0; 
    Ar(7,14)=(-0.40672*t16+40.672*t28)*x[18]; 
    Ar(7,15)=0; 
    Ar(7,16)=(-0.2032*t15+20.32*t27)*x[18]; 
    Ar(7,17)=0; 
    Ar(7,18)=(-0.40672*t16+40.672*t28)*x[14]+(-0.2032*t15+20.32*t27)*x[16]+1.62624*x[18]+(-0.2032*t17+20.32*t29)*x[20]; 
    Ar(7,19)=0; 
    Ar(7,20)=(-0.2032*t17+20.32*t29)*x[18]; 
    Ar(7,21)=0; 
    Ar(8,0)=0; 
    Ar(8,1)=0; 
    Ar(8,2)=0; 
    Ar(8,3)=0; 
    Ar(8,4)=0; 
    Ar(8,5)=(40.672*t13+0.40672*t25)*x[14]*x[19]; 
    Ar(8,6)=0; 
    Ar(8,7)=0; 
    Ar(8,8)=(20.32*t12+0.2032*t24)*x[17]*x[19]; 
    Ar(8,9)=0; 
    Ar(8,10)=x[19]*((-40.672*t13-0.40672*t25)*x[14]+(-20.32*t12-0.2032*t24)*x[17]+(-20.32*t14-0.2032*t26)*x[21]); 
    Ar(8,11)=0; 
    Ar(8,12)=(20.32*t14+0.2032*t26)*x[19]*x[21]; 
    Ar(8,13)=0; 
    Ar(8,14)=(-0.40672*t13+40.672*t25)*x[19]; 
    Ar(8,15)=0; 
    Ar(8,16)=0; 
    Ar(8,17)=(-0.2032*t12+20.32*t24)*x[19]; 
    Ar(8,18)=0; 
    Ar(8,19)=(-0.40672*t13+40.672*t25)*x[14]+(-0.2032*t12+20.32*t24)*x[17]+1.62624*x[19]+(-0.2032*t14+20.32*t26)*x[21]; 
    Ar(8,20)=0; 
    Ar(8,21)=(-0.2032*t14+20.32*t26)*x[19]; 
    Ar(9,0)=0; 
    Ar(9,1)=0; 
    Ar(9,2)=0; 
    Ar(9,3)=0; 
    Ar(9,4)=0; 
    Ar(9,5)=0; 
    Ar(9,6)=(40.672*t21-0.40672*t33)*x[15]*x[20]; 
    Ar(9,7)=0; 
    Ar(9,8)=0; 
    Ar(9,9)=(20.32*t17-0.2032*t29)*x[18]*x[20]; 
    Ar(9,10)=0; 
    Ar(9,11)=((-40.672*t21+0.40672*t33)*x[15]+(-20.32*t17+0.2032*t29)*x[18])*x[20]; 
    Ar(9,12)=0; 
    Ar(9,13)=0; 
    Ar(9,14)=0; 
    Ar(9,15)=(-0.40672*t21-40.672*t33)*x[20]; 
    Ar(9,16)=0; 
    Ar(9,17)=0; 
    Ar(9,18)=(-0.2032*t17-20.32*t29)*x[20]; 
    Ar(9,19)=0; 
    Ar(9,20)=(-0.40672*t21-40.672*t33)*x[15]+(-0.2032*t17-20.32*t29)*x[18]+1.21984*x[20]; 
    Ar(9,21)=0; 
    Ar(10,0)=0; 
    Ar(10,1)=0; 
    Ar(10,2)=0; 
    Ar(10,3)=0; 
    Ar(10,4)=0; 
    Ar(10,5)=0; 
    Ar(10,6)=(40.672*t20-0.40672*t32)*x[15]*x[21]; 
    Ar(10,7)=0; 
    Ar(10,8)=0; 
    Ar(10,9)=0; 
    Ar(10,10)=(20.32*t14-0.2032*t26)*x[19]*x[21]; 
    Ar(10,11)=0; 
    Ar(10,12)=((-40.672*t20+0.40672*t32)*x[15]+(-20.32*t14+0.2032*t26)*x[19])*x[21]; 
    Ar(10,13)=0; 
    Ar(10,14)=0; 
    Ar(10,15)=(-0.40672*t20-40.672*t32)*x[21]; 
    Ar(10,16)=0; 
    Ar(10,17)=0; 
    Ar(10,18)=0; 
    Ar(10,19)=(-0.2032*t14-20.32*t26)*x[21]; 
    Ar(10,20)=0; 
    Ar(10,21)=(-0.40672*t20-40.672*t32)*x[15]+(-0.2032*t14-20.32*t26)*x[19]+1.21984*x[21]; 
    Ar(11,0)=0; 
    Ar(11,1)=0; 
    Ar(11,2)=1.; 
    Ar(11,3)=0; 
    Ar(11,4)=-3.636*t35*u[2]*x[13]; 
    Ar(11,5)=0; 
    Ar(11,6)=0; 
    Ar(11,7)=0; 
    Ar(11,8)=0; 
    Ar(11,9)=0; 
    Ar(11,10)=0; 
    Ar(11,11)=0; 
    Ar(11,12)=0; 
    Ar(11,13)=-3.636*t23*u[2]; 
    Ar(11,14)=0; 
    Ar(11,15)=0; 
    Ar(11,16)=0; 
    Ar(11,17)=0; 
    Ar(11,18)=0; 
    Ar(11,19)=0; 
    Ar(11,20)=0; 
    Ar(11,21)=0; 
    Ar(12,0)=0; 
    Ar(12,1)=0; 
    Ar(12,2)=0; 
    Ar(12,3)=1.; 
    Ar(12,4)=0; 
    Ar(12,5)=-3.636*t34*u[3]*x[14]; 
    Ar(12,6)=0; 
    Ar(12,7)=0; 
    Ar(12,8)=0; 
    Ar(12,9)=0; 
    Ar(12,10)=0; 
    Ar(12,11)=0; 
    Ar(12,12)=0; 
    Ar(12,13)=0; 
    Ar(12,14)=-3.636*t22*u[3]; 
    Ar(12,15)=0; 
    Ar(12,16)=0; 
    Ar(12,17)=0; 
    Ar(12,18)=0; 
    Ar(12,19)=0; 
    Ar(12,20)=0; 
    Ar(12,21)=0; 
    Ar(13,0)=0; 
    Ar(13,1)=0; 
    Ar(13,2)=0; 
    Ar(13,3)=0; 
    Ar(13,4)=x[13]*(-3.636*t35*u[2]+(-0.40672*t19+40.672*t31)*x[16]+(-0.40672*t18+40.672*t30)*x[17]); 
    Ar(13,5)=0; 
    Ar(13,6)=0; 
    Ar(13,7)=(0.40672*t19-40.672*t31)*x[13]*x[16]; 
    Ar(13,8)=(0.40672*t18-40.672*t30)*x[13]*x[17]; 
    Ar(13,9)=0; 
    Ar(13,10)=0; 
    Ar(13,11)=0; 
    Ar(13,12)=0; 
    Ar(13,13)=-3.636*t23*u[2]+169.96*x[13]+(-40.672*t19-0.40672*t31)*x[16]+(-40.672*t18-0.40672*t30)*x[17]; 
    Ar(13,14)=0; 
    Ar(13,15)=0; 
    Ar(13,16)=(-40.672*t19-0.40672*t31)*x[13]; 
    Ar(13,17)=(-40.672*t18-0.40672*t30)*x[13]; 
    Ar(13,18)=0; 
    Ar(13,19)=0; 
    Ar(13,20)=0; 
    Ar(13,21)=0; 
    Ar(14,0)=0; 
    Ar(14,1)=0; 
    Ar(14,2)=0; 
    Ar(14,3)=0; 
    Ar(14,4)=0; 
    Ar(14,5)=x[14]*(-3.636*t34*u[3]+(-0.40672*t16+40.672*t28)*x[18]+(-0.40672*t13+40.672*t25)*x[19]); 
    Ar(14,6)=0; 
    Ar(14,7)=0; 
    Ar(14,8)=0; 
    Ar(14,9)=(0.40672*t16-40.672*t28)*x[14]*x[18]; 
    Ar(14,10)=(0.40672*t13-40.672*t25)*x[14]*x[19]; 
    Ar(14,11)=0; 
    Ar(14,12)=0; 
    Ar(14,13)=0; 
    Ar(14,14)=-3.636*t22*u[3]+169.96*x[14]+(-40.672*t16-0.40672*t28)*x[18]+(-40.672*t13-0.40672*t25)*x[19]; 
    Ar(14,15)=0; 
    Ar(14,16)=0; 
    Ar(14,17)=0; 
    Ar(14,18)=(-40.672*t16-0.40672*t28)*x[14]; 
    Ar(14,19)=(-40.672*t13-0.40672*t25)*x[14]; 
    Ar(14,20)=0; 
    Ar(14,21)=0; 
    Ar(15,0)=0; 
    Ar(15,1)=0; 
    Ar(15,2)=0; 
    Ar(15,3)=0; 
    Ar(15,4)=0; 
    Ar(15,5)=0; 
    Ar(15,6)=x[15]*((-0.40672*t21-40.672*t33)*x[20]+(-0.40672*t20-40.672*t32)*x[21]); 
    Ar(15,7)=0; 
    Ar(15,8)=0; 
    Ar(15,9)=0; 
    Ar(15,10)=0; 
    Ar(15,11)=(0.40672*t21+40.672*t33)*x[15]*x[20]; 
    Ar(15,12)=(0.40672*t20+40.672*t32)*x[15]*x[21]; 
    Ar(15,13)=0; 
    Ar(15,14)=0; 
    Ar(15,15)=169.96*x[15]+(-40.672*t21+0.40672*t33)*x[20]+(-40.672*t20+0.40672*t32)*x[21]; 
    Ar(15,16)=0; 
    Ar(15,17)=0; 
    Ar(15,18)=0; 
    Ar(15,19)=0; 
    Ar(15,20)=(-40.672*t21+0.40672*t33)*x[15]; 
    Ar(15,21)=(-40.672*t20+0.40672*t32)*x[15]; 
    Ar(16,0)=0; 
    Ar(16,1)=0; 
    Ar(16,2)=0; 
    Ar(16,3)=0; 
    Ar(16,4)=(0.40672*t19+40.672*t31)*x[13]*x[16]; 
    Ar(16,5)=0; 
    Ar(16,6)=0; 
    Ar(16,7)=x[16]*((-0.40672*t19-40.672*t31)*x[13]+(-0.2032*t15+20.32*t27)*x[18]); 
    Ar(16,8)=0; 
    Ar(16,9)=(0.2032*t15-20.32*t27)*x[16]*x[18]; 
    Ar(16,10)=0; 
    Ar(16,11)=0; 
    Ar(16,12)=0; 
    Ar(16,13)=(-40.672*t19+0.40672*t31)*x[16]; 
    Ar(16,14)=0; 
    Ar(16,15)=0; 
    Ar(16,16)=(-40.672*t19+0.40672*t31)*x[13]+121.984*x[16]+(-20.32*t15-0.2032*t27)*x[18]; 
    Ar(16,17)=0; 
    Ar(16,18)=(-20.32*t15-0.2032*t27)*x[16]; 
    Ar(16,19)=0; 
    Ar(16,20)=0; 
    Ar(16,21)=0; 
    Ar(17,0)=0; 
    Ar(17,1)=0; 
    Ar(17,2)=0; 
    Ar(17,3)=0; 
    Ar(17,4)=(0.40672*t18+40.672*t30)*x[13]*x[17]; 
    Ar(17,5)=0; 
    Ar(17,6)=0; 
    Ar(17,7)=0; 
    Ar(17,8)=x[17]*((-0.40672*t18-40.672*t30)*x[13]+(-0.2032*t12+20.32*t24)*x[19]); 
    Ar(17,9)=0; 
    Ar(17,10)=(0.2032*t12-20.32*t24)*x[17]*x[19]; 
    Ar(17,11)=0; 
    Ar(17,12)=0; 
    Ar(17,13)=(-40.672*t18+0.40672*t30)*x[17]; 
    Ar(17,14)=0; 
    Ar(17,15)=0; 
    Ar(17,16)=0; 
    Ar(17,17)=(-40.672*t18+0.40672*t30)*x[13]+121.984*x[17]+(-20.32*t12-0.2032*t24)*x[19]; 
    Ar(17,18)=0; 
    Ar(17,19)=(-20.32*t12-0.2032*t24)*x[17]; 
    Ar(17,20)=0; 
    Ar(17,21)=0; 
    Ar(18,0)=0; 
    Ar(18,1)=0; 
    Ar(18,2)=0; 
    Ar(18,3)=0; 
    Ar(18,4)=0; 
    Ar(18,5)=(0.40672*t16+40.672*t28)*x[14]*x[18]; 
    Ar(18,6)=0; 
    Ar(18,7)=(0.2032*t15+20.32*t27)*x[16]*x[18]; 
    Ar(18,8)=0; 
    Ar(18,9)=x[18]*((-0.40672*t16-40.672*t28)*x[14]+(-0.2032*t15-20.32*t27)*x[16]+(-0.2032*t17-20.32*t29)*x[20]); 
    Ar(18,10)=0; 
    Ar(18,11)=(0.2032*t17+20.32*t29)*x[18]*x[20]; 
    Ar(18,12)=0; 
    Ar(18,13)=0; 
    Ar(18,14)=(-40.672*t16+0.40672*t28)*x[18]; 
    Ar(18,15)=0; 
    Ar(18,16)=(-20.32*t15+0.2032*t27)*x[18]; 
    Ar(18,17)=0; 
    Ar(18,18)=(-40.672*t16+0.40672*t28)*x[14]+(-20.32*t15+0.2032*t27)*x[16]+162.624*x[18]+(-20.32*t17+0.2032*t29)*x[20]; 
    Ar(18,19)=0; 
    Ar(18,20)=(-20.32*t17+0.2032*t29)*x[18]; 
    Ar(18,21)=0; 
    Ar(19,0)=0; 
    Ar(19,1)=0; 
    Ar(19,2)=0; 
    Ar(19,3)=0; 
    Ar(19,4)=0; 
    Ar(19,5)=(0.40672*t13+40.672*t25)*x[14]*x[19]; 
    Ar(19,6)=0; 
    Ar(19,7)=0; 
    Ar(19,8)=(0.2032*t12+20.32*t24)*x[17]*x[19]; 
    Ar(19,9)=0; 
    Ar(19,10)=x[19]*((-0.40672*t13-40.672*t25)*x[14]+(-0.2032*t12-20.32*t24)*x[17]+(-0.2032*t14-20.32*t26)*x[21]); 
    Ar(19,11)=0; 
    Ar(19,12)=(0.2032*t14+20.32*t26)*x[19]*x[21]; 
    Ar(19,13)=0; 
    Ar(19,14)=(-40.672*t13+0.40672*t25)*x[19]; 
    Ar(19,15)=0; 
    Ar(19,16)=0; 
    Ar(19,17)=(-20.32*t12+0.2032*t24)*x[19]; 
    Ar(19,18)=0; 
    Ar(19,19)=(-40.672*t13+0.40672*t25)*x[14]+(-20.32*t12+0.2032*t24)*x[17]+162.624*x[19]+(-20.32*t14+0.2032*t26)*x[21]; 
    Ar(19,20)=0; 
    Ar(19,21)=(-20.32*t14+0.2032*t26)*x[19]; 
    Ar(20,0)=0; 
    Ar(20,1)=0; 
    Ar(20,2)=0; 
    Ar(20,3)=0; 
    Ar(20,4)=0; 
    Ar(20,5)=0; 
    Ar(20,6)=(0.40672*t21-40.672*t33)*x[15]*x[20]; 
    Ar(20,7)=0; 
    Ar(20,8)=0; 
    Ar(20,9)=(0.2032*t17-20.32*t29)*x[18]*x[20]; 
    Ar(20,10)=0; 
    Ar(20,11)=((-0.40672*t21+40.672*t33)*x[15]+(-0.2032*t17+20.32*t29)*x[18])*x[20]; 
    Ar(20,12)=0; 
    Ar(20,13)=0; 
    Ar(20,14)=0; 
    Ar(20,15)=(-40.672*t21-0.40672*t33)*x[20]; 
    Ar(20,16)=0; 
    Ar(20,17)=0; 
    Ar(20,18)=(-20.32*t17-0.2032*t29)*x[20]; 
    Ar(20,19)=0; 
    Ar(20,20)=(-40.672*t21-0.40672*t33)*x[15]+(-20.32*t17-0.2032*t29)*x[18]+121.984*x[20]; 
    Ar(20,21)=0; 
    Ar(21,0)=0; 
    Ar(21,1)=0; 
    Ar(21,2)=0; 
    Ar(21,3)=0; 
    Ar(21,4)=0; 
    Ar(21,5)=0; 
    Ar(21,6)=(0.40672*t20-40.672*t32)*x[15]*x[21]; 
    Ar(21,7)=0; 
    Ar(21,8)=0; 
    Ar(21,9)=0; 
    Ar(21,10)=(0.2032*t14-20.32*t26)*x[19]*x[21]; 
    Ar(21,11)=0; 
    Ar(21,12)=((-0.40672*t20+40.672*t32)*x[15]+(-0.2032*t14+20.32*t26)*x[19])*x[21]; 
    Ar(21,13)=0; 
    Ar(21,14)=0; 
    Ar(21,15)=(-40.672*t20-0.40672*t32)*x[21]; 
    Ar(21,16)=0; 
    Ar(21,17)=0; 
    Ar(21,18)=0; 
    Ar(21,19)=(-20.32*t14-0.2032*t26)*x[21]; 
    Ar(21,20)=0; 
    Ar(21,21)=(-40.672*t20-0.40672*t32)*x[15]+(-20.32*t14-0.2032*t26)*x[19]+121.984*x[21]; 
    Br(0)=3.636*t23*u[2]*x[4]*x[13]; 
    Br(1)=3.636*t22*u[3]*x[5]*x[14]; 
    Br(2)=0.81344*t7-u[4]-x[13]*(3.636*t23*u[2]*x[4]+((40.672*t19-0.40672*t31)*x[4]+t19*(0.40672-40.672*x[7])+t31*(40.672+0.40672*x[7]))*x[16]+((40.672*t18-0.40672*t30)*x[4]+t18*(0.40672-40.672*x[8])+t30*(40.672+0.40672*x[8]))*x[17]); 
    Br(3)=0.81344*t6-u[5]-x[14]*(3.636*t22*u[3]*x[5]+((40.672*t16-0.40672*t28)*x[5]+t16*(0.40672-40.672*x[9])+t28*(40.672+0.40672*x[9]))*x[18]+((40.672*t13-0.40672*t25)*x[5]+t13*(0.40672-40.672*x[10])+t25*(40.672+0.40672*x[10]))*x[19]); 
    Br(4)=0.81344*t5-u[6]-x[15]*(((40.672*t21+0.40672*t33)*x[6]+t21*(0.40672-40.672*x[11])+t33*(-40.672-0.40672*x[11]))*x[20]+((40.672*t20+0.40672*t32)*x[6]+t20*(0.40672-40.672*x[12])+t32*(-40.672-0.40672*x[12]))*x[21]); 
    Br(5)=0.60992*t4-u[7]-x[16]*((t19*(0.40672-40.672*x[4])+t31*(-40.672-0.40672*x[4])+(40.672*t19+0.40672*t31)*x[7])*x[13]+((20.32*t15-0.2032*t27)*x[7]+t15*(0.2032-20.32*x[9])+t27*(20.32+0.2032*x[9]))*x[18]); 
    Br(6)=0.60992*t3-u[8]-x[17]*((t18*(0.40672-40.672*x[4])+t30*(-40.672-0.40672*x[4])+(40.672*t18+0.40672*t30)*x[8])*x[13]+((20.32*t12-0.2032*t24)*x[8]+t12*(0.2032-20.32*x[10])+t24*(20.32+0.2032*x[10]))*x[19]); 
    Br(7)=0.81312*t2-u[9]-x[18]*((t16*(0.40672-40.672*x[5])+t28*(-40.672-0.40672*x[5]))*x[14]+(t15*(0.2032-20.32*x[7])+t27*(-20.32-0.2032*x[7]))*x[16]+(t17*(0.2032-20.32*x[11])+t29*(-20.32-0.2032*x[11]))*x[20]+x[9]*((40.672*t16+0.40672*t28)*x[14]+(20.32*t15+0.2032*t27)*x[16]+(20.32*t17+0.2032*t29)*x[20])); 
    Br(8)=0.81312*t1-u[10]-x[19]*((t13*(0.40672-40.672*x[5])+t25*(-40.672-0.40672*x[5]))*x[14]+(t12*(0.2032-20.32*x[8])+t24*(-20.32-0.2032*x[8]))*x[17]+(t14*(0.2032-20.32*x[12])+t26*(-20.32-0.2032*x[12]))*x[21]+x[10]*((40.672*t13+0.40672*t25)*x[14]+(20.32*t12+0.2032*t24)*x[17]+(20.32*t14+0.2032*t26)*x[21])); 
    Br(9)=0.60992*t10-u[11]-((t21*(0.40672-40.672*x[6])+t33*(40.672+0.40672*x[6])+(40.672*t21-0.40672*t33)*x[11])*x[15]+(t17*(0.2032-20.32*x[9])+t29*(20.32+0.2032*x[9])+(20.32*t17-0.2032*t29)*x[11])*x[18])*x[20]; 
    Br(10)=0.60992*t9-u[12]-((t20*(0.40672-40.672*x[6])+t32*(40.672+0.40672*x[6])+(40.672*t20-0.40672*t32)*x[12])*x[15]+(t14*(0.2032-20.32*x[10])+t26*(20.32+0.2032*x[10])+(20.32*t14-0.2032*t26)*x[12])*x[19])*x[21]; 
    Br(11)=-3.636*t11-3.636*t35*u[2]*x[4]*x[13]; 
    Br(12)=-3.636*t8-3.636*t34*u[3]*x[5]*x[14]; 
    Br(13)=84.98*t7-u[13]-x[13]*(3.636*t35*u[2]*x[4]+((0.40672*t19-40.672*t31)*x[4]+t19*(40.672-0.40672*x[7])+t31*(0.40672+40.672*x[7]))*x[16]+((0.40672*t18-40.672*t30)*x[4]+t18*(40.672-0.40672*x[8])+t30*(0.40672+40.672*x[8]))*x[17]); 
    Br(14)=84.98*t6-u[14]-x[14]*(3.636*t34*u[3]*x[5]+((0.40672*t16-40.672*t28)*x[5]+t16*(40.672-0.40672*x[9])+t28*(0.40672+40.672*x[9]))*x[18]+((0.40672*t13-40.672*t25)*x[5]+t13*(40.672-0.40672*x[10])+t25*(0.40672+40.672*x[10]))*x[19]); 
    Br(15)=84.98*t5-u[15]-x[15]*(((0.40672*t21+40.672*t33)*x[6]+t33*(-0.40672-40.672*x[11])+t21*(40.672-0.40672*x[11]))*x[20]+((0.40672*t20+40.672*t32)*x[6]+t32*(-0.40672-40.672*x[12])+t20*(40.672-0.40672*x[12]))*x[21]); 
    Br(16)=60.992*t4-u[16]-x[16]*((t31*(-0.40672-40.672*x[4])+t19*(40.672-0.40672*x[4])+(0.40672*t19+40.672*t31)*x[7])*x[13]+((0.2032*t15-20.32*t27)*x[7]+t15*(20.32-0.2032*x[9])+t27*(0.2032+20.32*x[9]))*x[18]); 
    Br(17)=60.992*t3-u[17]-x[17]*((t30*(-0.40672-40.672*x[4])+t18*(40.672-0.40672*x[4])+(0.40672*t18+40.672*t30)*x[8])*x[13]+((0.2032*t12-20.32*t24)*x[8]+t12*(20.32-0.2032*x[10])+t24*(0.2032+20.32*x[10]))*x[19]); 
    Br(18)=81.312*t2-u[18]-x[18]*((t28*(-0.40672-40.672*x[5])+t16*(40.672-0.40672*x[5]))*x[14]+(t27*(-0.2032-20.32*x[7])+t15*(20.32-0.2032*x[7]))*x[16]+(t29*(-0.2032-20.32*x[11])+t17*(20.32-0.2032*x[11]))*x[20]+x[9]*((0.40672*t16+40.672*t28)*x[14]+(0.2032*t15+20.32*t27)*x[16]+(0.2032*t17+20.32*t29)*x[20])); 
    Br(19)=81.312*t1-u[19]-x[19]*((t25*(-0.40672-40.672*x[5])+t13*(40.672-0.40672*x[5]))*x[14]+(t24*(-0.2032-20.32*x[8])+t12*(20.32-0.2032*x[8]))*x[17]+(t26*(-0.2032-20.32*x[12])+t14*(20.32-0.2032*x[12]))*x[21]+x[10]*((0.40672*t13+40.672*t25)*x[14]+(0.2032*t12+20.32*t24)*x[17]+(0.2032*t14+20.32*t26)*x[21])); 
    Br(20)=60.992*t10-u[20]-((t21*(40.672-0.40672*x[6])+t33*(0.40672+40.672*x[6])+(0.40672*t21-40.672*t33)*x[11])*x[15]+(t17*(20.32-0.2032*x[9])+t29*(0.2032+20.32*x[9])+(0.2032*t17-20.32*t29)*x[11])*x[18])*x[20]; 
    Br(21)=60.992*t9-u[21]-((t20*(40.672-0.40672*x[6])+t32*(0.40672+40.672*x[6])+(0.40672*t20-40.672*t32)*x[12])*x[15]+(t14*(20.32-0.2032*x[10])+t26*(0.2032+20.32*x[10])+(0.2032*t14-20.32*t26)*x[12])*x[19])*x[21]; 
    /* Solve linear system A*X = C*P + F  */ 
    nrhs = 1; 
    LDA = n; 
    int ipiv[n]; 
    dgesv_(&n,&nrhs,&Ar(0,0),&LDA,ipiv,&Br(0),&LDA,&info); 
    /* reassign results */ 
    tempx0 = Br(0); 
    tempx1 = Br(1); 
    tempx2 = Br(2); 
    tempx3 = Br(3); 
    tempx4 = Br(4); 
    tempx5 = Br(5); 
    tempx6 = Br(6); 
    tempx7 = Br(7); 
    tempx8 = Br(8); 
    tempx9 = Br(9); 
    tempx10 = Br(10); 
    tempx11 = Br(11); 
    tempx12 = Br(12); 
    tempx13 = Br(13); 
    tempx14 = Br(14); 
    tempx15 = Br(15); 
    tempx16 = Br(16); 
    tempx17 = Br(17); 
    tempx18 = Br(18); 
    tempx19 = Br(19); 
    tempx20 = Br(20); 
    tempx21 = Br(21); 
    x[0] = tempx0; 
    x[1] = tempx1; 
    x[2] = tempx2; 
    x[3] = tempx3; 
    x[4] = tempx4; 
    x[5] = tempx5; 
    x[6] = tempx6; 
    x[7] = tempx7; 
    x[8] = tempx8; 
    x[9] = tempx9; 
    x[10] = tempx10; 
    x[11] = tempx11; 
    x[12] = tempx12; 
    x[13] = tempx13; 
    x[14] = tempx14; 
    x[15] = tempx15; 
    x[16] = tempx16; 
    x[17] = tempx17; 
    x[18] = tempx18; 
    x[19] = tempx19; 
    x[20] = tempx20; 
    x[21] = tempx21; 
  } 
} 
/* 
 * mdlOutputs - compute the outputs 
 * 
 * In this function, you compute the outputs of your S-function 
 * block.  The outputs are placed in the y variable. 
 */ 
static void mdlOutputs(y, x, u, S, tid) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double *y, *x, *u; 
    SimStruct *S; 
    int tid; 
{ 
    double s1, s2, s3, s4, s5, s6, s7, s8, s9, s10; 
  
#else    /* compiled for dSpace board? */ 
    real_T *y; 
	  const real_T *x, *u; 
    SimStruct *S; 
    int_T tid; 
{ 
    real_T s1, s2, s3, s4, s5, s6, s7, s8, s9, s10; 
  
#endif 
    y[0]=x[0]; 
    y[1]=x[1]; 
    y[2]=x[2]; 
    y[3]=x[3]; 
    y[4]=x[4]; 
    y[5]=x[5]; 
    y[6]=x[6]; 
    y[7]=x[7]; 
    y[8]=x[8]; 
    y[9]=x[9]; 
    y[10]=x[10]; 
    y[11]=x[11]; 
    y[12]=x[12]; 
    y[13]=x[13]; 
    y[14]=x[14]; 
    y[15]=x[15]; 
    y[16]=x[16]; 
    y[17]=x[17]; 
    y[18]=x[18]; 
    y[19]=x[19]; 
    y[20]=x[20]; 
    y[21]=x[21]; 
} 
/* 
 * mdlDerivatives - compute the derivatives 
 * 
 * In this function, you compute the S-function block's derivatives. 
 * The derivatives are placed in the dx variable. 
 */ 
static void mdlDerivatives(dx, x, u, S, tid) 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
    double  *dx, *x, *u; 
    SimStruct *S; 
    int tid; 
#else    /* compiled for dSpace board? */ 
    real_T *dx, *x; 
    const real_T *u; 
    SimStruct *S; 
    int_T tid; 
#endif 
{ 
} 
/* 
 * mdlTerminate - called when the simulation is terminated. 
 * 
 * In this function, you should perform any actions that are necessary 
 * at the termination of a simulation.  For example, if memory was allocated 
 * in mdlInitializeConditions, this is the place to free it. 
 */ 
static void mdlTerminate(S) 
    SimStruct *S; 
{ 
} 
#include "linsolv.c"    /* includes linsolve routine dposv_ linear equation solver*/ 
#ifdef MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */ 
#include "simulink.c"      /* MEX-file interface mechanism */ 
#else 
#include "cg_sfun.h"       /* Code generation registration function */ 
#endif 
